#don't look here... that's cheating

































































































def egg():
	egg = "i told you there weren't any easter eggs!"
	print(egg)